#include "NEBULAConstructionMessenger.hh"
#include "NEBULAConstruction.hh"

#include "NEBULASimParameterReader.hh"

#include "G4UIdirectory.hh"
#include "G4UIcmdWithAString.hh"

NEBULAConstructionMessenger::NEBULAConstructionMessenger()
{
  std::cout<<"NEBULAConstructionMessenger"<<std::endl;

  fNEBULADirectory = new G4UIdirectory("/samurai/geometry/NEBULA/");
  fNEBULADirectory->SetGuidance("Modification Commands for NEBULA");

  fParameterFileNameCmd = new G4UIcmdWithAString("/samurai/geometry/NEBULA/ParameterFileName",this);
  fParameterFileNameCmd->SetGuidance("Parameter file name for NEBULA.");
  fParameterFileNameCmd->SetParameterName("ParaNEBULA",false);

  fDetectorParameterFileNameCmd = new G4UIcmdWithAString("/samurai/geometry/NEBULA/DetectorParameterFileName",this);
  fDetectorParameterFileNameCmd->SetGuidance("Parameter file name for Neutron Detectors.");
  fDetectorParameterFileNameCmd->SetParameterName("ParaNEBULA",false);

}

NEBULAConstructionMessenger::~NEBULAConstructionMessenger()
{
  delete fNEBULADirectory;
  delete fParameterFileNameCmd;
  delete fDetectorParameterFileNameCmd;
}

void NEBULAConstructionMessenger::SetNewValue(G4UIcommand* command, G4String  newValue )
{
  if( command == fParameterFileNameCmd ){
    NEBULASimParameterReader reader;
    reader.ReadNEBULAParameters(newValue.data());

  }else  if( command == fDetectorParameterFileNameCmd ){
    NEBULASimParameterReader reader;
    reader.ReadNEBULADetectorParameters(newValue.data());

  }
}


